package com.ruoyi.ss.app.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.app.domain.App;
import com.ruoyi.ss.app.domain.AppQuery;
import com.ruoyi.ss.app.domain.AppVO;
import com.ruoyi.ss.app.mapper.AppMapper;
import com.ruoyi.ss.app.service.AppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * APP信息Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-08
 */
@Service
public class AppServiceImpl implements AppService
{
    @Autowired
    private AppMapper appMapper;

    /**
     * 查询APP信息
     *
     * @param id APP信息主键
     * @return APP信息
     */
    @Override
    public AppVO selectAppById(Long id)
    {
        return appMapper.selectAppById(id);
    }

    /**
     * 查询APP信息列表
     *
     * @param app APP信息
     * @return APP信息
     */
    @Override
    public List<AppVO> selectAppList(AppQuery app)
    {
        return appMapper.selectAppList(app);
    }

    /**
     * 新增APP信息
     *
     * @param app APP信息
     * @return 结果
     */
    @Override
    public int insertApp(App app)
    {
        app.setCreateTime(DateUtils.getNowDate());
        return appMapper.insertApp(app);
    }

    /**
     * 修改APP信息
     *
     * @param app APP信息
     * @return 结果
     */
    @Override
    public int updateApp(App app)
    {
        return appMapper.updateApp(app);
    }

    /**
     * 批量删除APP信息
     *
     * @param ids 需要删除的APP信息主键
     * @return 结果
     */
    @Override
    public int deleteAppByIds(Long[] ids)
    {
        return appMapper.deleteAppByIds(ids);
    }

    /**
     * 删除APP信息信息
     *
     * @param id APP信息主键
     * @return 结果
     */
    @Override
    public int deleteAppById(Long id)
    {
        return appMapper.deleteAppById(id);
    }

    @Override
    public List<AppVO> selectByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        AppQuery query = new AppQuery();
        query.setIds(ids);
        return appMapper.selectAppList(query);
    }
}
