package com.ruoyi.ss.article.service;


import com.ruoyi.ss.article.domain.SmArticle;
import com.ruoyi.ss.article.domain.SmArticleQuery;
import com.ruoyi.ss.article.domain.enums.LicenceType;
import com.ruoyi.ss.article.domain.vo.ArticleMchHelpVO;

import java.util.List;

/**
 * 文章Service接口
 *
 * @author qiuzhenzhao
 * @date 2023-12-06
 */
public interface ISmArticleService
{
    /**
     * 查询文章
     *
     * @param articleId 文章主键
     * @return 文章
     */
    public SmArticle selectSmArticleByArticleId(Long articleId);

    /**
     * 查询文章列表
     *
     * @param smArticle 文章
     * @return 文章集合
     */
    public List<SmArticle> selectSmArticleList(SmArticleQuery smArticle);

    /**
     * 新增文章
     *
     * @param smArticle 文章
     * @return 结果
     */
    public int insertSmArticle(SmArticle smArticle);

    /**
     * 修改文章
     *
     * @param smArticle 文章
     * @return 结果
     */
    public int updateSmArticle(SmArticle smArticle);

    /**
     * 批量删除文章
     *
     * @param articleIds 需要删除的文章主键集合
     * @return 结果
     */
    public int deleteSmArticleByArticleIds(Long[] articleIds);

    /**
     * 删除文章信息
     *
     * @param articleId 文章主键
     * @return 结果
     */
    public int deleteSmArticleByArticleId(Long articleId);

    /**
     * 根据文章id获取标签列表
     *
     * @param articleId 文章主键
     * @return 结果
     */
    public String[] getTagList(Long articleId);

    /**
     * 根据类型获取协议
     * @param type 协议类型
     */
    SmArticle selectLicenceByType(LicenceType type);

    /**
     * 获取商户帮助中心文章
     */
    List<ArticleMchHelpVO> selectMchHelp(SmArticleQuery query);
}
