package com.ruoyi.web.controller.ss;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.vipLevel.domain.VipLevelQuery;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevel.service.VipLevelAssembler;
import com.ruoyi.ss.vipLevel.service.VipLevelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Collections;
import java.util.List;

/**
 * 会员等级Controller
 *
 * @author ruoyi
 * @date 2024-12-09
 */
@RestController
@RequestMapping("/ss/vipLevel")
public class VipLevelController extends BaseController
{
    @Autowired
    private VipLevelService vipLevelService;

    @Autowired
    private VipLevelAssembler vipLevelAssembler;

    /**
     * 查询会员等级列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:list')")
    @GetMapping("/list")
    public TableDataInfo list(VipLevelQuery query)
    {
        startPage();
        startOrderBy();
        List<VipLevelVO> list = vipLevelService.selectVipLevelList(query);
        return getDataTable(list);
    }

    /**
     * 查询会员等级列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:list')")
    @PostMapping("/listByIds")
    public AjaxResult list(@RequestBody List<Long> ids)
    {
        VipLevelQuery query = new VipLevelQuery();
        query.setIds(ids);
        List<VipLevelVO> list = vipLevelService.selectVipLevelList(query);
        return success(list);
    }

    /**
     * 导出会员等级列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:export')")
    @Log(title = "会员等级", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, VipLevelQuery query)
    {
        List<VipLevelVO> list = vipLevelService.selectVipLevelList(query);
        ExcelUtil<VipLevelVO> util = new ExcelUtil<VipLevelVO>(VipLevelVO.class);
        util.exportExcel(response, list, "会员等级数据");
    }

    /**
     * 获取会员等级详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        VipLevelVO level = vipLevelService.selectVipLevelById(id);
        List<VipLevelVO> list = Collections.singletonList(level);
        vipLevelAssembler.assembleSkuList(list);
        return success(level);
    }

    /**
     * 新增会员等级
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:add')")
    @Log(title = "会员等级", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) VipLevelVO vipLevel)
    {
        return toAjax(vipLevelService.insertVipLevel(vipLevel));
    }

    /**
     * 修改会员等级
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:edit')")
    @Log(title = "会员等级", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) VipLevelVO vipLevel)
    {
        return toAjax(vipLevelService.updateVipLevel(vipLevel));
    }

    /**
     * 删除会员等级
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevel:remove')")
    @Log(title = "会员等级", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(vipLevelService.deleteVipLevelByIds(ids));
    }
}
