package com.ruoyi.web.controller.ss;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSku;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuQuery;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.service.VipLevelSkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * VIP等级定价Controller
 *
 * @author ruoyi
 * @date 2025-01-16
 */
@RestController
@RequestMapping("/ss/vipLevelSku")
public class VipLevelSkuController extends BaseController
{
    @Autowired
    private VipLevelSkuService vipLevelSkuService;

    /**
     * 查询VIP等级定价列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:list')")
    @GetMapping("/list")
    public TableDataInfo list(VipLevelSkuQuery query)
    {
        startPage();
        startOrderBy();
        List<VipLevelSkuVO> list = vipLevelSkuService.selectVipLevelSkuList(query);
        return getDataTable(list);
    }

    /**
     * 导出VIP等级定价列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:export')")
    @Log(title = "VIP等级定价", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, VipLevelSkuQuery query)
    {
        List<VipLevelSkuVO> list = vipLevelSkuService.selectVipLevelSkuList(query);
        ExcelUtil<VipLevelSkuVO> util = new ExcelUtil<VipLevelSkuVO>(VipLevelSkuVO.class);
        util.exportExcel(response, list, "VIP等级定价数据");
    }

    /**
     * 获取VIP等级定价详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(vipLevelSkuService.selectVipLevelSkuById(id));
    }

    /**
     * 新增VIP等级定价
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:add')")
    @Log(title = "VIP等级定价", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody VipLevelSku vipLevelSku)
    {
        return toAjax(vipLevelSkuService.insertVipLevelSku(vipLevelSku));
    }

    /**
     * 修改VIP等级定价
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:edit')")
    @Log(title = "VIP等级定价", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody VipLevelSku vipLevelSku)
    {
        return toAjax(vipLevelSkuService.updateVipLevelSku(vipLevelSku));
    }

    /**
     * 删除VIP等级定价
     */
    @PreAuthorize("@ss.hasPermi('ss:vipLevelSku:remove')")
    @Log(title = "VIP等级定价", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(vipLevelSkuService.deleteVipLevelSkuByIds(ids));
    }
}
